@@ -563,10 +563,10 @@ int mpeg3_get_ps_pes_packet(mpeg3_demuxer_t *demuxer, unsigned int *header)
 			if(demuxer->astream == -1) 
 				demuxer->astream = stream_id & 0x0f;
 
+			demuxer->pes_audio_time = demuxer->time;
+			if(pts) demuxer->pes_audio_time = pts;
 			if((stream_id & 0x0f) == demuxer->astream && demuxer->do_audio)
 			{
-				if(pts) demuxer->pes_audio_time = pts;
-
 				memcpy(&demuxer->data_buffer[demuxer->data_size],
 					&demuxer->raw_data[demuxer->raw_offset],
 					pes_packet_length);
@@ -589,10 +589,11 @@ int mpeg3_get_ps_pes_packet(mpeg3_demuxer_t *demuxer, unsigned int *header)
 			if(demuxer->vstream == -1) 
 				demuxer->vstream = stream_id & 0x0f;
 
+       		demuxer->pes_video_time  = demuxer->time;;
+       		if(pts) demuxer->pes_video_time = pts;
 			pes_packet_length -= demuxer->raw_offset - pes_packet_start;
     	    if((stream_id & 0x0f) == demuxer->vstream && demuxer->do_video)
 			{
-        		if(pts) demuxer->pes_video_time = pts;
 
 				memcpy(&demuxer->data_buffer[demuxer->data_size], 
 					&demuxer->raw_data[demuxer->raw_offset],
@@ -607,11 +608,13 @@ int mpeg3_get_ps_pes_packet(mpeg3_demuxer_t *demuxer, unsigned int *header)
     	}
     	else 
 		if((stream_id == 0xbd || stream_id == 0xbf) && 
-			mpeg3packet_next_char(demuxer) != 0xff)
+			mpeg3packet_next_char(demuxer) != 0xff &&
+			((mpeg3packet_next_char(demuxer) & 0xf0) != 0x20))
 		{
 /* DVD audio data */
 /* Get the audio format */
 			int format;
+//printf("mpeg3_get_ps_pes_packet 5 %x\n", mpeg3packet_next_char(demuxer) & 0xf0);
 			if((mpeg3packet_next_char(demuxer) & 0xf0) == 0xa0)
 				format = AUDIO_PCM;
 			else
@@ -619,6 +622,10 @@ int mpeg3_get_ps_pes_packet(mpeg3_demuxer_t *demuxer, unsigned int *header)
 
 // Picks up bogus data if (& 0xf) or (& 0x7f)
 			stream_id = mpeg3packet_next_char(demuxer);
+			demuxer->pes_audio_time = demuxer->time;
+       		if(pts) demuxer->pes_audio_time = pts;
+
+//printf("mpeg3_get_ps_pes_packet %x\n", stream_id);
 /* Take first stream ID if not building TOC. */
 			if(demuxer->read_all)
 				demuxer->astream_table[stream_id] = format;
@@ -629,7 +636,6 @@ int mpeg3_get_ps_pes_packet(mpeg3_demuxer_t *demuxer, unsigned int *header)
       		if(stream_id == demuxer->astream && demuxer->do_audio)
 			{
 				demuxer->aformat = format;
-        		if(pts) demuxer->pes_audio_time = pts;
 				mpeg3packet_read_int32(demuxer);
 				pes_packet_length -= demuxer->raw_offset - pes_packet_start;
 
@@ -644,6 +650,7 @@ int mpeg3_get_ps_pes_packet(mpeg3_demuxer_t *demuxer, unsigned int *header)
 				pes_packet_length -= demuxer->raw_offset - pes_packet_start;
         	    mpeg3packet_skip(demuxer, pes_packet_length);
       		}
+//printf("mpeg3_get_ps_pes_packet 6 %d\n", demuxer->astream_table[0x20]);
     	}
     	else 
 		if(stream_id == 0xbc || 1)
@@ -676,6 +683,7 @@ int mpeg3_read_program(mpeg3_demuxer_t *demuxer)
 	result = mpeg3io_eof(title->fs);
 
 	if(!result) result = mpeg3io_seek_relative(title->fs, -4);
+//printf("mpeg3_read_program 1 %lx\n", mpeg3io_tell(title->fs));
 
 // Search backwards for header
 	while(header != MPEG3_PACK_START_CODE && !result && count < demuxer->packet_size)
@@ -697,6 +705,7 @@ int mpeg3_read_program(mpeg3_demuxer_t *demuxer)
 	}
 
 	result = mpeg3io_read_data(demuxer->raw_data, demuxer->packet_size, title->fs);
+//printf("mpeg3_read_program 2 %d %lx\n", demuxer->packet_size, mpeg3io_tell(title->fs));
 
 	if(result)
 	{
@@ -734,6 +743,7 @@ int mpeg3_read_program(mpeg3_demuxer_t *demuxer)
 		else
 			demuxer->raw_offset -= 3;
 	}
+//printf("mpeg3_read_program 3\n");
 	return result;
 }
 
@@ -752,35 +762,33 @@ double mpeg3_lookup_time_offset(mpeg3_demuxer_t *demuxer, long byte)
 	return title->timecode_table[i].absolute_start_time - title->timecode_table[i].start_time;
 }
 
-int mpeg3_advance_timecode(mpeg3_demuxer_t *demuxer, int reverse)
+int mpeg3_advance_timecode(mpeg3_demuxer_t *demuxer)
 {
 	mpeg3_title_t *title = demuxer->titles[demuxer->current_title];
 	int result = 0;
 	int do_seek = 0;
+	int last_timecode = demuxer->current_timecode;
+//printf("mpeg3_advance_timecode 0\n");fflush(stdout);
 
 /* Skip timecode advancing when constructing timecode table */
 	if(!title->timecode_table || 
 		!title->timecode_table_size || 
 		demuxer->read_all) return 0;
 
-//printf("mpeg3_advance_timecode 1\n");
-	if(!reverse)
+//printf("mpeg3_advance_timecode 1\n");fflush(stdout);
+	if(!demuxer->reverse)
 	{
 /* Get inside the current timecode */
 		if(mpeg3io_tell(title->fs) < title->timecode_table[demuxer->current_timecode].start_byte)
 		{
 			mpeg3io_seek(title->fs, title->timecode_table[demuxer->current_timecode].start_byte);
 		}
-		
+
 /* Get the next timecode */
 		while(!result && 
 			(mpeg3io_tell(title->fs) >= title->timecode_table[demuxer->current_timecode].end_byte ||
 				demuxer->current_program != title->timecode_table[demuxer->current_timecode].program))
 		{
-
-//printf("mpeg3_advance_timecode 1 %d %d %d %d\n", mpeg3io_tell(title->fs), title->timecode_table[demuxer->current_timecode].end_byte,
-//	demuxer->current_program, title->timecode_table[demuxer->current_timecode].program);
-
 			demuxer->current_timecode++;
 			if(demuxer->current_timecode >= title->timecode_table_size)
 			{
@@ -799,6 +807,9 @@ int mpeg3_advance_timecode(mpeg3_demuxer_t *demuxer, int reverse)
 			title = demuxer->titles[demuxer->current_title];
 		}
 
+//if(last_timecode != demuxer->current_timecode && demuxer->do_video)
+//	printf("using title %d timecode %x %x\n", demuxer->current_title, title->timecode_table[demuxer->current_timecode].start_byte, title->timecode_table[demuxer->current_timecode].end_byte);
+
 //printf("2 %d\n", title->timecode_table[demuxer->current_timecode].program);
 		if(!result && do_seek)
 		{
@@ -813,16 +824,24 @@ int mpeg3_advance_timecode(mpeg3_demuxer_t *demuxer, int reverse)
 				demuxer->current_program != title->timecode_table[demuxer->current_timecode].program))
 		{
 /*
- * if(demuxer->do_audio) printf("mpeg3_reverse_timecode %d %d %d %d\n", mpeg3io_tell(title->fs), title->timecode_table[demuxer->current_timecode].end_byte,
- * 	demuxer->current_program, title->timecode_table[demuxer->current_timecode].program);
+ * printf("mpeg3_reverse_timecode %d %d %d %d\n", 
+ * 	mpeg3io_tell(title->fs), 
+ * 	demuxer->current_timecode,
+ * 	title->timecode_table[demuxer->current_timecode].start_byte,
+ *  title->timecode_table[demuxer->current_timecode].end_byte);
  */
+
 			demuxer->current_timecode--;
 			if(demuxer->current_timecode < 0)
 			{
 				if(demuxer->current_title > 0)
 				{
-					mpeg3demux_open_title(demuxer, demuxer->current_title - 1);
+//printf("advance_timecode 2 %d\n", demuxer->current_title);
+					mpeg3demux_open_title(demuxer, --demuxer->current_title);
 					title = demuxer->titles[demuxer->current_title];
+// Seek to end since we opened at the beginning of the next title
+					mpeg3io_seek(title->fs, title->total_bytes - demuxer->packet_size);
+//printf("advance_timecode 3 %d %d\n", demuxer->current_title, mpeg3io_tell(title->fs));
 					demuxer->current_timecode = title->timecode_table_size - 1;
 					do_seek = 1;
 				}
@@ -839,6 +858,7 @@ int mpeg3_advance_timecode(mpeg3_demuxer_t *demuxer, int reverse)
 			mpeg3io_seek(title->fs, title->timecode_table[demuxer->current_timecode].start_byte);
 	}
 
+//printf("mpeg3_advance_timecode 2 %d\n", demuxer->current_title);fflush(stdout);
 	return result;
 }
 
@@ -852,8 +872,8 @@ int mpeg3_read_next_packet(mpeg3_demuxer_t *demuxer)
 	demuxer->data_size = 0;
 	demuxer->data_position = 0;
 
-/* Flip the file descriptor back to the end of the packet for forward */
-/* reading. */
+//printf("mpeg3_read_next_packet 1 %d %lx\n", demuxer->current_title, mpeg3io_tell(title->fs));
+/* Switch to forward direction. */
 	if(demuxer->reverse)
 	{
 		result = mpeg3io_seek_relative(title->fs, demuxer->packet_size);
@@ -865,7 +885,7 @@ int mpeg3_read_next_packet(mpeg3_demuxer_t *demuxer)
 	{
 		do
 		{
-			result = mpeg3_advance_timecode(demuxer, 0);
+			result = mpeg3_advance_timecode(demuxer);
 
 			if(!result)
 			{
@@ -887,11 +907,13 @@ int mpeg3_read_next_packet(mpeg3_demuxer_t *demuxer)
 					if(!result) demuxer->data_size = demuxer->packet_size;
 				}
 			}
+//printf("mpeg3_read_next_packet 2 %d %lx\n", demuxer->current_title, mpeg3io_tell(title->fs));
 		}while(!result && 
 			demuxer->data_size == 0 && 
 			(demuxer->do_audio || demuxer->do_video));
 	}
 
+//printf("mpeg3_read_next_packet 4\n");
 	return result;
 }
 
@@ -902,31 +924,38 @@ int mpeg3_read_prev_packet(mpeg3_demuxer_t *demuxer)
 	mpeg3_t *file = demuxer->file;
 	long current_position;
 	mpeg3_title_t *title = demuxer->titles[demuxer->current_title];
-
 	demuxer->data_size = 0;
 	demuxer->data_position = 0;
 
+/* Switch to reverse direction */
+	if(!demuxer->reverse)
+	{
+		result = mpeg3io_seek_relative(title->fs, -demuxer->packet_size);
+		demuxer->reverse = 1;
+	}
+
+//printf("mpeg3_read_prev_packet 1 %d %d\n", demuxer->current_title, demuxer->current_timecode);
 	do
 	{
-/* Rewind to the start of the packet to be read. */
+// Go to beginning of previous packet
 		result = mpeg3io_seek_relative(title->fs, -demuxer->packet_size);
-
-		if(!result) result = mpeg3_advance_timecode(demuxer, 1);
+//printf("mpeg3_read_prev_packet 1 %p\n", title->fs->fd);
+		if(!result) result = mpeg3_advance_timecode(demuxer);
+//printf("mpeg3_read_prev_packet 2 %p->%p->%p\n", title, title->fs, title->fs->fd);
 		if(!result) demuxer->time_offset = mpeg3_lookup_time_offset(demuxer, mpeg3io_tell(title->fs));
 
 		if(file->is_transport_stream && !result)
 		{
 			result = mpeg3_read_transport(demuxer);
 			if(!mpeg3io_bof(title->fs))
-			/* if(!result)  */result = mpeg3io_seek_relative(title->fs, -demuxer->packet_size);
+				result = mpeg3io_seek_relative(title->fs, -demuxer->packet_size);
 		}
 		else
 		if(file->is_program_stream && !result)
 		{
-
 			result = mpeg3_read_program(demuxer);
 			if(!mpeg3io_bof(title->fs))
-			/* if(!result)  */result = mpeg3io_seek_relative(title->fs, -demuxer->packet_size);
+				result = mpeg3io_seek_relative(title->fs, -demuxer->packet_size);
 		}
 		else
 		if(!result)
@@ -940,11 +969,10 @@ int mpeg3_read_prev_packet(mpeg3_demuxer_t *demuxer)
 				result = mpeg3io_seek_relative(title->fs, -demuxer->packet_size);
 			}
 		}
-	}while(!result && demuxer->data_size == 0 && (demuxer->do_audio || demuxer->do_video));
-
-/* Remember that the file descriptor is at the beginning of the packet just read. */
-	demuxer->reverse = 1;
-	demuxer->error_flag = result;
+	}while(!result && 
+		demuxer->data_size == 0 && 
+		(demuxer->do_audio || demuxer->do_video));
+//printf("mpeg3_read_prev_packet 2 %d %d\n", demuxer->current_title, demuxer->current_timecode);
 	return result;
 }
 
@@ -1102,7 +1130,7 @@ int mpeg3demux_open_title(mpeg3_demuxer_t *demuxer, int title_number)
 		}
 	}
 
-	demuxer->current_timecode = 0;
+//	demuxer->current_timecode = 0;
 
 	return demuxer->error_flag;
 }
@@ -1179,6 +1207,7 @@ int mpeg3demux_copy_titles(mpeg3_demuxer_t *dst, mpeg3_demuxer_t *src)
 	}
 
 	mpeg3demux_open_title(dst, src->current_title);
+	dst->current_timecode = 0;
 	return 0;
 }
 
@@ -1277,6 +1306,15 @@ int mpeg3demux_bof(mpeg3_demuxer_t *demuxer)
 	return 0;
 }
 
+void mpeg3demux_start_reverse(mpeg3_demuxer_t *demuxer)
+{
+	demuxer->reverse = 1;
+}
+
+void mpeg3demux_start_forward(mpeg3_demuxer_t *demuxer)
+{
+	demuxer->reverse = 0;
+}
 
 /* Seek to a byte in the current title */
 int mpeg3demux_seek_byte(mpeg3_demuxer_t *demuxer, long byte)
@@ -1305,12 +1343,14 @@ int mpeg3demux_seek_byte(mpeg3_demuxer_t *demuxer, long byte)
 int mpeg3demux_seek_end(mpeg3_demuxer_t *demuxer)
 {
 	mpeg3demux_open_title(demuxer, demuxer->total_titles - 1);
+	demuxer->current_timecode = 0;
 	return mpeg3demux_seek_byte(demuxer, demuxer->titles[demuxer->current_title]->total_bytes);
 }
 
 int mpeg3demux_seek_start(mpeg3_demuxer_t *demuxer)
 {
 	mpeg3demux_open_title(demuxer, 0);
+	demuxer->current_timecode = 0;
 	return mpeg3demux_seek_byte(demuxer, 0);
 }
 
@@ -1463,12 +1503,26 @@ int mpeg3demux_seek_percentage(mpeg3_demuxer_t *demuxer, double percentage)
 		demuxer->current_timecode++;
 	}
 
+/*
+ * printf("seek percentage 1 %d %d %d %d\n", demuxer->current_title, 
+ * 	demuxer->current_timecode,
+ * 	title->timecode_table[demuxer->current_timecode].start_byte,
+ * 	title->timecode_table[demuxer->current_timecode].end_byte);
+ */
+
 /* Open the new title and seek to the correct byte */
 	if(new_title != demuxer->current_title)
 	{
 		demuxer->error_flag = mpeg3demux_open_title(demuxer, new_title);
 	}
 
+/*
+ * printf("seek percentage 2 %d %d %d %d\n", demuxer->current_title, 
+ * 	demuxer->current_timecode,
+ * 	title->timecode_table[demuxer->current_timecode].start_byte,
+ * 	title->timecode_table[demuxer->current_timecode].end_byte);
+ */
+
 	if(!demuxer->error_flag)
 		demuxer->error_flag = mpeg3io_seek(title->fs, relative_position);
 
