@@ -27,6 +27,7 @@
  *
  */
 
+#define MAX(a,b) ( (a)>(b) ? (a) : (b) )
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -46,27 +47,6 @@ int main(argc,argv)
 int argc;
 char *argv[];
 {
-  if(argc < 2)
-  {
-    printf("mpeg2encode V1.3, 2000/01/10\n"
-"(C) 1996, MPEG Software Simulation Group\n"
-"(C) 2000 Adam Williams\n"
-"Usage: encode [-p] [-b bitrate] [-1] [-q quant] [-n N] [-m M] <input file> <output file>\n\n"
-" -b bitrate                         fix the bitrate, vary the quantization\n"
-" -p                                              encode progressive frames\n"
-" -1                            generate an MPEG-1 stream instead of MPEG-2\n"
-" -q quantization                    fix the quantization, vary the bitrate\n"
-" -n frames                           set number of frames between I frames\n"
-" -m frames                           set number of frames between P frames\n\n"
-"Default settings:\n"
-"   fixed 5000000 bits/sec\n"
-"   interlaced\n"
-"   MPEG-2\n"
-"   15 frames between I frames   0 frames between P frames\n\n"
-"For the recommended encoding parameters see docs/index.html.\n");
-    exit(1);
-  }
-
 /* Read command line */
 	readcmdline(argc, argv);
 
@@ -86,14 +66,18 @@ char *argv[];
 	}
 
 	init();
+
+	printf("Frame    Completion    Current bitrate     Predicted file size\n");
 	putseq();
 
 	stop_slice_engines();
 	stop_motion_engines();
   	stop_transform_engines();
+  	stop_itransform_engines();
 	fclose(outfile);
 	fclose(statfile);
 	if(qt_file) quicktime_close(qt_file);
+	if(qt_output) quicktime_close(qt_output);
 	if(mpeg_file) mpeg3_close(mpeg_file);
 
 	return 0;
@@ -135,14 +119,44 @@ int VertMotionCode(int i)
 	return 5;
 }
 
+/*
+	Wrapper for malloc that allocates pbuffers aligned to the 
+	specified byte boundary and checks for failure.
+	N.b.  don't try to free the resulting pointers, eh...
+	BUG: 	Of course this won't work if a char * won't fit in an int....
+*/
+static uint8_t *bufalloc( size_t size )
+{
+	char *buf = malloc( size + BUFFER_ALIGN );
+	int adjust;
+
+	if( buf == NULL )
+	{
+		error("malloc failed\n");
+	}
+	adjust = BUFFER_ALIGN-((int)buf)%BUFFER_ALIGN;
+	if( adjust == BUFFER_ALIGN )
+		adjust = 0;
+	return (uint8_t*)(buf+adjust);
+}
+
 static void init()
 {
-	int i, size;
+	int i, n, size;
 	static int block_count_tab[3] = {6,8,12};
+	int lum_buffer_size, chrom_buffer_size;
+	pthread_mutexattr_t mutex_attr;
+	pthread_mutexattr_init(&mutex_attr);
+	pthread_mutex_init(&test_lock, &mutex_attr);
 
+	bzero(&cur_picture, sizeof(pict_data_s));
 	initbits();
 	init_fdct();
 	init_idct();
+	init_motion();
+	init_predict();
+	init_quantizer();
+	init_transform();
 
 /* round picture dimensions to nearest multiple of 16 or 32 */
 	mb_width = (horizontal_size+15)/16;
@@ -159,6 +173,18 @@ static void init()
 	chrom_width2 = fieldpic ? chrom_width<<1 : chrom_width;
 
 	block_count = block_count_tab[chroma_format-1];
+	lum_buffer_size = (width*height) + 
+					 sizeof(uint8_t) *(width/2)*(height/2) +
+					 sizeof(uint8_t) *(width/4)*(height/4+1);
+	chrom_buffer_size = chrom_width*chrom_height;
+
+	fsubsample_offset = (width)*(height) * sizeof(uint8_t);
+	qsubsample_offset =  fsubsample_offset + (width/2)*(height/2)*sizeof(uint8_t);
+
+	rowsums_offset = 0;
+	colsums_offset = 0;
+
+	mb_per_pict = mb_width*mb_height2;
 
 /* clip table */
 	if (!(clp = (unsigned char *)malloc(1024)))
@@ -167,37 +193,50 @@ static void init()
 	for (i=-384; i<640; i++)
       clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);
 
-	for (i=0; i<3; i++)
-	{
-      size = (i==0) ? width*height : chrom_width*chrom_height;
 
-      if (!(newrefframe[i] = (unsigned char *)malloc(size)))
-    	error("malloc failed\n");
-      if (!(oldrefframe[i] = (unsigned char *)malloc(size)))
-    	error("malloc failed\n");
-      if (!(auxframe[i] = (unsigned char *)malloc(size)))
-    	error("malloc failed\n");
-      if (!(neworgframe[i] = (unsigned char *)malloc(size)))
-    	error("malloc failed\n");
-      if (!(oldorgframe[i] = (unsigned char *)malloc(size)))
-    	error("malloc failed\n");
-      if (!(auxorgframe[i] = (unsigned char *)malloc(size)))
-    	error("malloc failed\n");
-      if (!(predframe[i] = (unsigned char *)malloc(size)))
-    	error("malloc failed\n");
+	
+	/* Allocate the frame buffer */
+
+
+	frame_buffers = (uint8_t ***) 
+		bufalloc(2*READ_LOOK_AHEAD*sizeof(uint8_t**));
+	
+	for(n=0;n<2*READ_LOOK_AHEAD;n++)
+	{
+         frame_buffers[n] = (uint8_t **) bufalloc(3*sizeof(uint8_t*));
+		 for (i=0; i<3; i++)
+		 {
+			 frame_buffers[n][i] = 
+				 bufalloc( (i==0) ? lum_buffer_size : chrom_buffer_size );
+		 }
 	}
 
-	mbinfo = (struct mbinfo *)malloc(mb_width * mb_height2 * sizeof(struct mbinfo));
 
-	if (!mbinfo)
-      error("malloc failed\n");
 
-	blocks =
-      (short (*)[64])malloc(mb_width * mb_height2 * block_count * sizeof(short [64]));
+	/* TODO: The ref and aux frame buffers are no redundant! */
+	for( i = 0 ; i<3; i++)
+	{
+		int size =  (i==0) ? lum_buffer_size : chrom_buffer_size;
+		newrefframe[i] = bufalloc(size);
+		oldrefframe[i] = bufalloc(size);
+		auxframe[i]    = bufalloc(size);
+		predframe[i]   = bufalloc(size);
+	}
 
-	if (!blocks)
-      error("malloc failed\n");
+	cur_picture.qblocks =
+		(int16_t (*)[64])bufalloc(mb_per_pict*block_count*sizeof(int16_t [64]));
 
+	/* Initialise current transformed picture data tables
+	   These will soon become a buffer for transformed picture data to allow
+	   look-ahead for bit allocation etc.
+	 */
+	cur_picture.mbinfo = (
+		struct mbinfo *)bufalloc(mb_per_pict*sizeof(struct mbinfo));
+
+	cur_picture.blocks =
+		(int16_t (*)[64])bufalloc(mb_per_pict * block_count * sizeof(int16_t [64]));
+  
+  
 /* open statistics output file */
 	if(statname[0]=='-') statfile = stdout;
 	else 
@@ -207,10 +246,16 @@ static void init()
       error(errortext);
 	}
 
-	ratectl = calloc(1, sizeof(ratectl_t));
+	ratectl = malloc(processors * sizeof(ratectl_t*));
+	for(i = 0; i < processors; i++)
+		ratectl[i] = calloc(1, sizeof(ratectl_t));
+	
+
+
 /* Start parallel threads */
 	start_motion_engines();
 	start_transform_engines();
+	start_itransform_engines();
 	start_slice_engines();
 }
 
@@ -224,6 +269,42 @@ char *text;
 
 #define STRINGLEN 254
 
+int calculate_smp()
+{
+/* Get processor count */
+	int result = 1;
+	FILE *proc;
+	if(proc = fopen("/proc/cpuinfo", "r"))
+	{
+		char string[1024];
+		while(!feof(proc))
+		{
+			fgets(string, 1024, proc);
+			if(!strncasecmp(string, "processor", 9))
+			{
+				char *ptr = strchr(string, ':');
+				if(ptr)
+				{
+					ptr++;
+					result = atol(ptr) + 1;
+				}
+			}
+			else
+			if(!strncasecmp(string, "cpus detected", 13))
+			{
+				char *ptr = strchr(string, ':');
+				if(ptr)
+				{
+					ptr++;
+					result = atol(ptr);
+				}
+			}
+		}
+		fclose(proc);
+	}
+	return result;
+}
+
 static void readcmdline(int argc, char *argv[])
 {
 	int i;
@@ -252,46 +333,65 @@ static void readcmdline(int argc, char *argv[])
 	int vbvlim[4] = { 597, 448, 112, 29 };
 	long total_frame_rates = (sizeof(ratetab) / sizeof(double));
 	FILE *proc;
+	int param_searchrad = 16;
 
+	use_hires_quant = 0;
+	use_denoise_quant = 0;
 	quiet = 1;
 	bit_rate = 5000000;                       /* default bit_rate (bits/s) */
 	prog_seq = 0;                        /* progressive_sequence is faster */
 	mpeg1 = 0;                                   /* ISO/IEC 11172-2 stream */
     fixed_mquant = 0;                             /* vary the quantization */
+	quant_floor = 0;
+	act_boost = 3.0;
 	N = 15;                                      /* N (# of frames in GOP) */
 	M = 1;  /* M (I/P frame distance) */
-	processors = 1;
+	processors = calculate_smp();
 	frame_rate = -1;
 
-/* Get processor count */
-	if(proc = fopen("/proc/cpuinfo", "r"))
-	{
-		char string[1024];
-		while(!feof(proc))
-		{
-			fgets(string, 1024, proc);
-			if(!strncasecmp(string, "processor", 9))
-			{
-				char *ptr = strchr(string, ':');
-				if(ptr)
-				{
-					ptr++;
-					processors = atol(ptr) + 1;
-				}
-			}
-		}
-		fclose(proc);
-	}
-	else
-	{
-		fprintf(stderr, "Can't open /proc/cpuinfo.\n");
-	}
 
 	sprintf(tplorg, "");
 	sprintf(out_path, "");
 
+#define INTTOYES(x) ((x) ? "Yes" : "No")
+  if(argc < 2)
+  {
+    printf("mpeg2encode V1.3, 2000/01/10\n"
+"(C) 1996, MPEG Software Simulation Group\n"
+"(C) 2000 Adam Williams\n"
+"Usage: encode [options] <input file> <output file>\n\n"
+" -1                 generate an MPEG-1 stream instead of MPEG-2 (%s)\n"
+" -b bitrate              fix the bitrate, vary the quantization (%d)\n"
+" -d                                                     Denoise (%s)\n"
+" -h                          High resolution quantization table (%s)\n"
+" -m frames                set number of frames between P frames (%d)\n"
+" -n frames                set number of frames between I frames (%d)\n"
+" -p                                   encode progressive frames (%s)\n"
+" -q quantization         fix the quantization, vary the bitrate\n"
+" -u                                        Use only 1 processor\n\n"
+"Default settings:\n"
+"   fixed 5000000 bits/sec\n"
+"   interlaced\n"
+"   MPEG-2\n"
+"   15 frames between I frames   0 frames between P frames\n\n"
+"For the recommended encoding parameters see docs/index.html.\n",
+mpeg1 ? "MPEG-1" : "MPEG-2",
+(int)bit_rate,
+INTTOYES(use_denoise_quant),
+INTTOYES(use_hires_quant),
+M - 1,
+N,
+INTTOYES(prog_seq));
+    exit(1);
+  }
+
 	for(i = 1; i < argc; i++)
 	{
+		if(!strcmp(argv[i], "-1"))
+		{
+			mpeg1 = 1;
+		}
+		else
 		if(!strcmp(argv[i], "-b"))
 		{
 			i++;
@@ -306,16 +406,40 @@ static void readcmdline(int argc, char *argv[])
 			}
 		}
 		else
-		if(!strcmp(argv[i], "-q"))
+		if(!strcmp(argv[i], "-d"))
+		{
+			use_denoise_quant = 1;
+		}
+		else
+		if(!strcmp(argv[i], "-f"))
 		{
 			i++;
 			if(i < argc)
 			{
-				fixed_mquant = atol(argv[i]);
+				frame_rate = atof(argv[i]);
 			}
 			else
 			{
-				fprintf(stderr, "-q requires a quantization value\n");
+				fprintf(stderr, "-f requires a frame rate\n");
+				exit(1);
+			}
+		}
+		else
+		if(!strcmp(argv[i], "-h"))
+		{
+			use_hires_quant = 1;
+		}
+		else
+		if(!strcmp(argv[i], "-m"))
+		{
+			i++;
+			if(i < argc)
+			{
+				M = atol(argv[i]) + 1;
+			}
+			else
+			{
+				fprintf(stderr, "-m requires a frame count\n");
 				exit(1);
 			}
 		}
@@ -334,42 +458,28 @@ static void readcmdline(int argc, char *argv[])
 			}
 		}
 		else
-		if(!strcmp(argv[i], "-m"))
-		{
-			i++;
-			if(i < argc)
-			{
-				M = atol(argv[i]) + 1;
-			}
-			else
-			{
-				fprintf(stderr, "-m requires a frame count\n");
-				exit(1);
-			}
-		}
-		else
-		if(!strcmp(argv[i], "-f"))
-		{
-			i++;
-			if(i < argc)
-			{
-				frame_rate = atof(argv[i]);
-			}
-			else
-			{
-				fprintf(stderr, "-f requires a frame rate\n");
-				exit(1);
-			}
-		}
-		else
 		if(!strcmp(argv[i], "-p"))
 		{
 			prog_seq = 1;
 		}
 		else
-		if(!strcmp(argv[i], "-1"))
+		if(!strcmp(argv[i], "-q"))
 		{
-			mpeg1 = 1;
+			i++;
+			if(i < argc)
+			{
+				fixed_mquant = atol(argv[i]);
+			}
+			else
+			{
+				fprintf(stderr, "-q requires a quantization value\n");
+				exit(1);
+			}
+		}
+		else
+		if(!strcmp(argv[i], "-u"))
+		{
+			processors = 1;
 		}
 		else
 		if(!strlen(tplorg))
@@ -460,7 +570,7 @@ static void readcmdline(int argc, char *argv[])
 	}
 	frame0 =                   0;  /* number of first frame */
 	h = m = s = f =            0;  /* timecode of first frame */
-	fieldpic =                 0;  /* 0:frame pictures, 1:field pictures */
+	fieldpic =                 0;  /* 0: progressive, 1: bottom first, 2: top first, 3 = progressive seq, field MC and DCT in picture */
 	aspectratio =              1;  /* aspect_ratio_information 1=square pel, 2=4:3, 3=16:9, 4=2.11:1 */
 	low_delay =                0;  /* low_delay  */
 	constrparms =              0;  /* constrained_parameters_flag */
@@ -469,47 +579,62 @@ static void readcmdline(int argc, char *argv[])
 	chroma_format =            1;  /* chroma_format: 1=4:2:0, 2=4:2:2, 3=4:4:4   LibMPEG3 only does 1 */
 	video_format =             2;  /* video_format: 0=comp., 1=PAL, 2=NTSC, 3=SECAM, 4=MAC, 5=unspec. */
 	color_primaries =          5;  /* color_primaries */
+	dctsatlim		= mpeg1 ? 255 : 2047;
+	dctsatlim = 255;
 	transfer_characteristics = 5;  /* transfer_characteristics */
 	matrix_coefficients =      4;  /* matrix_coefficients (not used) */
 	display_horizontal_size =  horizontal_size;
 	display_vertical_size =    vertical_size;
-	dc_prec =                  0;  /* intra_dc_precision (0: 8 bit, 1: 9 bit, 2: 10 bit, 3: 11 bit */
-	topfirst =                 1;  /* top_field_first */
-	frame_pred_dct_tab[0] =    0;  /* frame_pred_frame_dct (I P B) */
-	frame_pred_dct_tab[1] =    0;  /* frame_pred_frame_dct (I P B) */
-	frame_pred_dct_tab[2] =    0;  /* frame_pred_frame_dct (I P B) */
-	conceal_tab[0] =           0;  /* concealment_motion_vectors (I P B) */
-	conceal_tab[1] =           0;  /* concealment_motion_vectors (I P B) */
-	conceal_tab[2] =           0;  /* concealment_motion_vectors (I P B) */
-	qscale_tab[0] =            1;  /* q_scale_type  (I P B) */
-	qscale_tab[1] =            1;  /* q_scale_type  (I P B) */
-	qscale_tab[2] =            1;  /* q_scale_type  (I P B) */
-	intravlc_tab[0] =          1;  /* intra_vlc_format (I P B)*/
-	intravlc_tab[1] =          0;  /* intra_vlc_format (I P B)*/
-	intravlc_tab[2] =          0;  /* intra_vlc_format (I P B)*/
-	altscan_tab[0] =           0;  /* alternate_scan (I P B) */
-	altscan_tab[1] =           0;  /* alternate_scan (I P B) */
-	altscan_tab[2] =           0;  /* alternate_scan (I P B) */
-	repeatfirst =              0;  /* repeat_first_field */
-	prog_frame =               0;  /* progressive_frame */
+	cur_picture.dc_prec =      0;  /* intra_dc_precision (0: 8 bit, 1: 9 bit, 2: 10 bit, 3: 11 bit */
+	cur_picture.topfirst =     1;  /* top_field_first */
+
+	frame_pred_dct_tab[0] =    mpeg1 ? 1 : 0;  /* frame_pred_frame_dct (I P B) */
+	frame_pred_dct_tab[1] =    mpeg1 ? 1 : 0;  /* frame_pred_frame_dct (I P B) */
+	frame_pred_dct_tab[2] =    mpeg1 ? 1 : 0;  /* frame_pred_frame_dct (I P B) */
+
+	conceal_tab[0]  		 = 0;  /* concealment_motion_vectors (I P B) */
+	conceal_tab[1]  		 = 0;  /* concealment_motion_vectors (I P B) */
+	conceal_tab[2]  		 = 0;  /* concealment_motion_vectors (I P B) */
+	qscale_tab[0]   		 = mpeg1 ? 0 : 1;  /* q_scale_type  (I P B) */
+	qscale_tab[1]   		 = mpeg1 ? 0 : 1;  /* q_scale_type  (I P B) */
+	qscale_tab[2]   		 = mpeg1 ? 0 : 1;  /* q_scale_type  (I P B) */
+
+	intravlc_tab[0] 		 = 0;  /* intra_vlc_format (I P B)*/
+	intravlc_tab[1] 		 = 0;  /* intra_vlc_format (I P B)*/
+	intravlc_tab[2] 		 = 0;  /* intra_vlc_format (I P B)*/
+	altscan_tab[0]  		 = 0;  /* alternate_scan (I P B) */
+	altscan_tab[1]  		 = 0;  /* alternate_scan (I P B) */
+	altscan_tab[2]  		 = 0;  /* alternate_scan (I P B) */
+	opt_dc_prec         = 0;  /* 8 bits */
+	opt_topfirst        = (fieldpic == 2);
+	opt_repeatfirst     = 0;
+	opt_prog_frame      = prog_seq;
+	cur_picture.repeatfirst =              0;  /* repeat_first_field */
+	cur_picture.prog_frame =               prog_seq;  /* progressive_frame */
 /* P:  forw_hor_f_code forw_vert_f_code search_width/height */
     motion_data = (struct motion_data *)malloc(3 * sizeof(struct motion_data));
+	video_buffer_size =        46 * 1024 * 8;
 
 /************************************************************************
  *                                END PARAMETER FILE
  ************************************************************************/
 
-	if(mpeg1) prog_seq = 1;
+	if(mpeg1)
+	{
+		opt_prog_frame = 1;
+		cur_picture.prog_frame = 1;
+		prog_seq = 1;
+	}
+
+	if(qt_file)
+		input_frame_rate = quicktime_frame_rate(qt_file, 0);
+	else
+	if(mpeg_file)
+		input_frame_rate = mpeg3_frame_rate(mpeg_file, 0);
 
 	if(frame_rate < 0)
 	{
-		if(qt_file)
-		{
-			frame_rate = quicktime_frame_rate(qt_file, 0);
-		}
-		else
-		if(mpeg_file)
-			frame_rate = mpeg3_frame_rate(mpeg_file, 0);
+		frame_rate = input_frame_rate;
 	}
 
 //processors = 1;
@@ -527,6 +652,8 @@ static void readcmdline(int argc, char *argv[])
 	printf("   %s\n", (mpeg1 ? "MPEG-1" : "MPEG-2"));
 	printf("   %d processors\n", processors);
 	printf("   %.02f frames per second\n", frame_rate);
+	printf("   Denoise %s\n", INTTOYES(use_denoise_quant));
+	printf("   Hires quantization %s\n", INTTOYES(use_hires_quant));
 
 
 	if(mpeg_file)
@@ -538,25 +665,55 @@ static void readcmdline(int argc, char *argv[])
 
 
 
+	{ 
+		int radius_x = ((param_searchrad+4)/8)*8;
+		int radius_y = ((param_searchrad*vertical_size/horizontal_size+4)/8)*8;
+		int c;
+	
+		/* TODO: These f-codes should really be adjusted for each
+		   picture type... */
+		c=5;
+		if( radius_x*M < 64) c = 4;
+		if( radius_x*M < 32) c = 3;
+		if( radius_x*M < 16) c = 2;
+		if( radius_x*M < 8) c = 1;
 
-/* Auto settings (from bbMPEG http://members.home.net/beyeler/bbmpeg.html) */
-    for(i = 1; i < M; i++)
-    {
-       motion_data[i].sxf = 3 * i;
-       motion_data[i].forw_hor_f_code = HorzMotionCode(motion_data[i].sxf);
-       motion_data[i].syf = 3 * i;
-       motion_data[i].forw_vert_f_code = VertMotionCode(motion_data[i].syf);
-       motion_data[i].sxb = 3 * (M - 1);
-       motion_data[i].back_hor_f_code = HorzMotionCode(motion_data[i].sxb);
-       motion_data[i].syb = 3 * (M - 1);
-       motion_data[i].back_vert_f_code = VertMotionCode(motion_data[i].syb);
-    }
-	motion_data[0].sxf = 3 * M;
-	motion_data[0].forw_hor_f_code = HorzMotionCode(motion_data[0].sxf);
-	motion_data[0].syf = 3 * M;
-	motion_data[0].forw_vert_f_code = VertMotionCode(motion_data[0].syf);
+		if (!motion_data)
+			error("malloc failed\n");
 
-    vbv_buffer_size = floor(((double)bit_rate * 0.20343) / 16384.0);
+		for (i=0; i<M; i++)
+		{
+			if(i==0)
+			{
+				motion_data[i].forw_hor_f_code  = c;
+				motion_data[i].forw_vert_f_code = c;
+				motion_data[i].sxf = MAX(1,radius_x*M);
+				motion_data[i].syf = MAX(1,radius_y*M);
+			}
+			else
+			{
+				motion_data[i].forw_hor_f_code  = c;
+				motion_data[i].forw_vert_f_code = c;
+				motion_data[i].sxf = MAX(1,radius_x*i);
+				motion_data[i].syf = MAX(1,radius_y*i);
+				motion_data[i].back_hor_f_code  = c;
+				motion_data[i].back_vert_f_code = c;
+				motion_data[i].sxb = MAX(1,radius_x*(M-i));
+				motion_data[i].syb = MAX(1,radius_y*(M-i));
+			}
+		}
+		
+	}
+
+//    vbv_buffer_size = floor(((double)bit_rate * 0.20343) / 16384.0);
+    if(mpeg1)
+		vbv_buffer_size = 20 * 16384;
+	else
+		vbv_buffer_size = 112 * 16384;
+
+	fast_mc_frac    = 10;
+	mc_44_red		= 2;
+	mc_22_red		= 3;
 
     if(vbv_buffer_size > vbvlim[(level - 4) >> 1])
         vbv_buffer_size = vbvlim[(level - 4) >> 1];
@@ -578,7 +735,7 @@ static void readcmdline(int argc, char *argv[])
   low_delay = !!low_delay;
   constrparms = !!constrparms;
   prog_seq = !!prog_seq;
-  topfirst = !!topfirst;
+  cur_picture.topfirst = !!cur_picture.topfirst;
 
   for (i = 0; i < 3; i++)
   {
@@ -588,8 +745,8 @@ static void readcmdline(int argc, char *argv[])
     intravlc_tab[i] = !!intravlc_tab[i];
     altscan_tab[i] = !!altscan_tab[i];
   }
-  repeatfirst = !!repeatfirst;
-  prog_frame = !!prog_frame;
+  cur_picture.repeatfirst = !!cur_picture.repeatfirst;
+  cur_picture.prog_frame = !!cur_picture.prog_frame;
 
   /* make sure MPEG specific parameters are valid */
   range_checks();
@@ -600,6 +757,27 @@ static void readcmdline(int argc, char *argv[])
   tc0 = 60*tc0 + s;
   tc0 = (int)(frame_rate+0.5)*tc0 + f;
 
+qt_output = 0;
+/*
+ * qt_output = quicktime_open("/mov/test.mov", 0, 1);
+ * quicktime_set_video(qt_output, 
+ * 	1, 
+ * 	horizontal_size, 
+ * 	vertical_size, 
+ * 	frame_rate, 
+ * 	QUICKTIME_YUV420);
+ */
+
+
+
+
+
+
+
+
+
+
+
   if (!mpeg1)
   {
     profile_and_level_checks();
@@ -620,39 +798,6 @@ static void readcmdline(int argc, char *argv[])
         constrparms = 0;
       }
     }
-
-    if (constrparms)
-    {
-      for (i=0; i<M; i++)
-      {
-        if (motion_data[i].forw_hor_f_code>4)
-        {
-          constrparms = 0;
-          break;
-        }
-
-        if (motion_data[i].forw_vert_f_code>4)
-        {
-          constrparms = 0;
-          break;
-        }
-
-        if (i!=0)
-        {
-          if (motion_data[i].back_hor_f_code>4)
-          {
-            constrparms = 0;
-            break;
-          }
-
-          if (motion_data[i].back_vert_f_code>4)
-          {
-            constrparms = 0;
-            break;
-          }
-        }
-      }
-    }
   }
 
   /* relational checks */
@@ -669,9 +814,9 @@ static void readcmdline(int argc, char *argv[])
       chroma_format = CHROMA420;
     }
 
-    if (dc_prec!=0)
+    if (cur_picture.dc_prec!=0)
     {
-      dc_prec = 0;
+      cur_picture.dc_prec = 0;
     }
 
     for (i=0; i<3; i++)
@@ -698,22 +843,22 @@ static void readcmdline(int argc, char *argv[])
     constrparms = 0;
   }
 
-  if (prog_seq && !prog_frame)
+  if (prog_seq && !cur_picture.prog_frame)
   {
-    prog_frame = 1;
+    cur_picture.prog_frame = 1;
   }
 
-  if (prog_frame && fieldpic)
+  if (cur_picture.prog_frame && fieldpic)
   {
     fieldpic = 0;
   }
 
-  if (!prog_frame && repeatfirst)
+  if (!cur_picture.prog_frame && cur_picture.repeatfirst)
   {
-    repeatfirst = 0;
+    cur_picture.repeatfirst = 0;
   }
 
-  if (prog_frame)
+  if (cur_picture.prog_frame)
   {
     for (i=0; i<3; i++)
       if (!frame_pred_dct_tab[i])
@@ -722,11 +867,11 @@ static void readcmdline(int argc, char *argv[])
       }
   }
 
-  if (prog_seq && !repeatfirst && topfirst)
+  if (prog_seq && !cur_picture.repeatfirst && cur_picture.topfirst)
   {
      if (!quiet)
        fprintf(stderr,"Warning: setting top_field_first = 0\n");
-    topfirst = 0;
+    cur_picture.topfirst = 0;
   }
 
   /* search windows */
@@ -774,64 +919,105 @@ static void readcmdline(int argc, char *argv[])
 
 }
 
+/*
+  If the user has selected suppression of hf noise via
+  quantisation then we boost quantisation of hf components
+  EXPERIMENTAL: currently a linear ramp from 0 at 4pel to 
+  50% increased quantisation...
+*/
+
+static int quant_hfnoise_filt(int orgquant, int qmat_pos )
+{
+	int x = qmat_pos % 8;
+	int y = qmat_pos / 8;
+	int qboost = 1024;
+
+	if(!use_denoise_quant)
+	{
+		return orgquant;
+	}
+
+	/* Maximum 50% quantisation boost for HF components... */
+	if( x > 4 )
+		qboost += (256*(x-4)/3);
+	if( y > 4 )
+		qboost += (256*(y-4)/3);
+
+	return (orgquant * qboost + 512)/ 1024;
+}
+
 static void readquantmat()
 {
-  int i,v;
-  FILE *fd;
+  int i,v,q;
+	load_iquant = 0;
+	load_niquant = 0;
 
   if (iqname[0]=='-')
   {
-    /* use default intra matrix */
-    load_iquant = 0;
-    for (i=0; i<64; i++)
-      intra_q[i] = default_intra_quantizer_matrix[i];
-  }
-  else
-  {
-    /* read customized intra matrix */
-    load_iquant = 1;
-    if (!(fd = fopen(iqname,"r")))
-    {
-      sprintf(errortext,"Couldn't open quant matrix file %s",iqname);
-      error(errortext);
-    }
-
-    for (i=0; i<64; i++)
-    {
-      fscanf(fd,"%d",&v);
-      if (v<1 || v>255)
-        error("invalid value in quant matrix");
-      intra_q[i] = v;
-    }
-
-    fclose(fd);
+  	if(use_hires_quant)
+	{
+		load_iquant |= 1;
+		for (i=0; i<64; i++)
+		{
+			intra_q[i] = hires_intra_quantizer_matrix[i];
+		}	
+	}
+	else
+	{
+		load_iquant = use_denoise_quant;
+		for (i=0; i<64; i++)
+		{
+			v = quant_hfnoise_filt(default_intra_quantizer_matrix[i], i);
+			if (v<1 || v>255)
+				error("value in intra quant matrix invalid (after noise filt adjust)");
+			intra_q[i] = v;
+		} 
+	}
   }
 
+/* TODO: Inv Quant matrix initialisation should check if the fraction fits in 16 bits! */
   if (niqname[0]=='-')
   {
-    /* use default non-intra matrix */
-    load_niquant = 0;
-    for (i=0; i<64; i++)
-      inter_q[i] = 16;
+		if(use_hires_quant)
+		{
+			for (i=0; i<64; i++)
+			{
+				inter_q[i] = hires_nonintra_quantizer_matrix[i];
+			}	
+		}
+		else
+		{
+/* default non-intra matrix is all 16's. For *our* default we use something
+   more suitable for domestic analog sources... which is non-standard...*/
+			load_niquant |= 1;
+			for (i=0; i<64; i++)
+			{
+				v = quant_hfnoise_filt(default_nonintra_quantizer_matrix[i],i);
+				if (v<1 || v>255)
+					error("value in non-intra quant matrix invalid (after noise filt adjust)");
+				inter_q[i] = v;
+			}
+		}
   }
-  else
-  {
-    /* read customized non-intra matrix */
-    load_niquant = 1;
-    if (!(fd = fopen(niqname,"r")))
-    {
-      sprintf(errortext,"Couldn't open quant matrix file %s",niqname);
-      error(errortext);
-    }
 
-    for (i=0; i<64; i++)
-    {
-      fscanf(fd,"%d",&v);
-      if (v<1 || v>255)
-        error("invalid value in quant matrix");
-      inter_q[i] = v;
-    }
-
-    fclose(fd);
-  }
+	for (i=0; i<64; i++)
+	{
+		i_intra_q[i] = (int)(((double)IQUANT_SCALE) / ((double)intra_q[i]));
+		i_inter_q[i] = (int)(((double)IQUANT_SCALE) / ((double)inter_q[i]));
+	}
+	
+	for( q = 1; q <= 112; ++q )
+	{
+		for (i=0; i<64; i++)
+		{
+			intra_q_tbl[q][i] = intra_q[i] * q;
+			inter_q_tbl[q][i] = inter_q[i] * q;
+			intra_q_tblf[q][i] = (float)intra_q_tbl[q][i];
+			inter_q_tblf[q][i] = (float)inter_q_tbl[q][i];
+			i_intra_q_tblf[q][i] = 1.0f/ ( intra_q_tblf[q][i] * 0.98);
+			i_intra_q_tbl[q][i] = (IQUANT_SCALE/intra_q_tbl[q][i]);
+			i_inter_q_tblf[q][i] =  1.0f/ (inter_q_tblf[q][i] * 0.98);
+			i_inter_q_tbl[q][i] = (IQUANT_SCALE/inter_q_tbl[q][i] );
+		}
+	}
 }
